<?php

declare(strict_types=1);

namespace Drupal\whatsapp_business\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use GuzzleHttp\Client;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a WhatsApp Business Test OTP form.
 */
final class LoginWithWhatsapp extends FormBase {

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Constructs the LoginWithWhatsapp.
   *
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger service.
   */
  public function __construct(LoggerInterface $logger) {
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('logger.channel.whatsapp_business')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'whatsapp_business_test_message';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $this->logger->info('Building the test message form.');

    $form['test_number'] = [
      '#type' => 'textfield',
      '#default_value' => '',
      '#title' => $this->t('Test Number'),
      '#placeholder' => $this->t('91xxxxxxxxxx'),
      '#description' => $this->t('Please note: The test number should already be registered in the WhatsApp Business platform to work.'),
      '#required' => TRUE,
    ];

    $form['test_otp'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('OTP Value'),
      '#description' => $this->t('Enter the 6 Digit OTP Value, Make sure Authentication template is activated in WA Manager https://business.facebook.com/latest/whatsapp_manager/message_templates'),
      '#default_value' => '123456',
    ];

    // Send OTP button inside the fieldset.
    $form['mobile_number_fieldset']['actions']['send_otp'] = [
      '#type' => 'submit',
      '#value' => $this->t('Send OTP'),
      '#submit' => ['::sendOtpSubmit'],
      '#limit_validation_errors' => [['mobile_number']],
      '#weight' => -5,
    ];

    // OTP fieldset appears after OTP is sent.
    if ($form_state->get('otp_sent')) {
      $form['otp_fieldset'] = [
        '#type' => 'fieldset',
        '#title' => $this->t('Verify OTP'),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
      ];

      // OTP input field inside the fieldset.
      $form['otp_fieldset']['otp'] = [
        '#type' => 'textfield',
        '#title' => $this->t('One-Time Password (OTP)'),
        '#description' => $this->t('Enter the OTP sent to your mobile number.'),
        '#required' => TRUE,
        '#maxlength' => 6,
        '#weight' => 0,
      ];

      // Submit button to validate OTP.
      $form['otp_fieldset']['actions']['validate_otp'] = [
        '#type' => 'submit',
        '#value' => $this->t('Submit'),
        '#weight' => 5,
      ];
    }

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Send'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    $test_number = $form_state->getValue('test_number');
    $this->logger->debug('Validating form with test number: @test_number', ['@test_number' => $test_number]);

    if (!preg_match('/^\d{12}$/', $test_number)) {
      $this->logger->warning('Validation failed: Invalid test number format.');
      $form_state->setErrorByName('test_number', $this->t('The test number must be a valid 12-digit number.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    // Read dynamic values from configuration.
    $config = $this->config('whatsapp_business.settings');

    $api_version = $config->get('wab_api_version');
    $phone_number_id = $config->get('wab_phone_number_id');
    $access_token = $config->get('wab_access_token');

    $default_template = $config->get('wab_default_template');
    $template_lang_code = $config->get('wab_default_template_lang_code');
    $template_variables = $config->get('wab_default_template_variables');

    // var_dump($template_lang_code);
    // die;
    // Get form values.
    $test_number = $form_state->getValue('test_number');
    $otp = $form_state->getValue('test_otp');

    // Log the settings and form values for debugging.
    \Drupal::logger('whatsapp_business')->debug('Configuration values: <pre><code>@config</pre></code>', [
      '@config' => json_encode([
        'api_version' => $api_version,
        'phone_number_id' => $phone_number_id,
        'default_template' => $default_template,
        'template_lang_code' => $template_lang_code,
      ]),
    ]);
    \Drupal::logger('whatsapp_business')->debug('Form values: <pre><code>@form_values</pre></code>', [
      '@form_values' => json_encode([
        'test_number' => $test_number,
        'otp' => $otp,
      ]),
    ]);

    try {
      $client = new Client();

      // Construct the message payload dynamically.
      $payload = [
        'messaging_product' => 'whatsapp',
        'recipient_type' => 'individual',
      // Dynamic phone number from form input.
        'to' => $test_number,
        'type' => 'template',
        'template' => [
      // Template name.
          'name' => $default_template,
          'language' => [
      // Dynamic language code (e.g., 'en')
            'code' => $template_lang_code,
          ],
          'components' => [
            [
              'type' => 'body',
              'parameters' => [
                [
                  'type' => 'text',
          // Dynamic OTP value.
                  'text' => $otp,
                ],
              ],
            ],
            [
              'type' => 'button',
              'sub_type' => 'url',
              'index' => 0,
              'parameters' => [
                [
                  'type' => 'text',
                  'text' => $otp,
                ],
              ],
            ],
          ],
        ],
      ];
      // var_dump($payload['template']['components']['0']['parameters']);
      // var_dump($this->buildTemplateParameters($template_variables, $otp));
      // die;
      // Log the payload for debugging.
      \Drupal::logger('whatsapp_business')->debug('Payload: {@payload}', [
        '@payload' => json_encode($payload, JSON_PRETTY_PRINT),
      ]);

      $response = $client->post("https://graph.facebook.com/{$api_version}/{$phone_number_id}/messages", [
        'headers' => [
          'Content-Type' => 'application/json',
          'Authorization' => "Bearer {$access_token}",
        ],
        'json' => $payload,
      ]);

      $status = $response->getStatusCode();
      $message = $status === 200 ? 'Message sent successfully!' : 'Message sent but not acknowledged.';

      // Log the response for debugging.
      \Drupal::logger('whatsapp_business')->info('Response status: <pre><code>@status</pre></code>', ['@status' => $status]);
      $this->messenger()->addStatus($this->t($message));
    }
    catch (\Exception $e) {
      $error_message = $e->getMessage();

      // Log the error for debugging.
      \Drupal::logger('whatsapp_business')->error('Failed to send the message. Error: @error', ['@error' => $error_message]);

      $this->messenger()->addError($this->t('Failed to send the message. Error: @error', ['@error' => $error_message]));
    }
  }

}
