<?php

declare(strict_types=1);

namespace Drupal\whatsapp_business\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\user\Entity\Role;

/**
 * Configure Whatsapp business settings for this site.
 */
final class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'whatsapp_business_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['whatsapp_business.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {

    $form['general_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('General Settings'),
      '#open' => TRUE,
      '#attributes' => [
        'class' => ['non-collapsible'],
      ],
    ];

    // Determine the default role.
    $default_user_role = $this->config('whatsapp_business.settings')->get('wab_user_roles') ?? 'authenticated';
    // Load all available user roles.
    $roles = Role::loadMultiple();
    $role_options = [];
    foreach ($roles as $role_id => $role) {
      // Exclude the 'anonymous' role from the list.
      if ($role_id !== 'anonymous') {
        $role_options[$role_id] = $role->label();
      }
    }

    // Add roles select field.
    $form['general_settings']['wab_user_roles'] = [
      '#type' => 'select',
      '#title' => $this->t('Select User Role'),
      '#description' => $this->t('Select default user role for the logging in user'),
      '#options' => $role_options,
      '#default_value' => $default_user_role,
      '#required' => TRUE,
    ];

    $base_url = Url::fromRoute('<front>', [], ['absolute' => TRUE])->toString();
    $parsed_url = parse_url($base_url);
    $site_domain = $parsed_url['host'];

    $otp_user_domain = $this->config('whatsapp_business.settings')->get('otp_user_domain');
    if (empty($otp_user_domain)) {
      $base_url = Url::fromRoute('<front>', [], ['absolute' => TRUE])->toString();
      $parsed_url = parse_url($base_url);
      $otp_user_domain = $parsed_url['host'];
    }

    $form['general_settings']['otp_user_domain'] = [
      '#type' => 'textfield',
      '#title' => $this->t('OTP User Domain'),
      '#default_value' => $this->config('whatsapp_business.settings')->get('otp_user_domain') ?? $site_domain,
      '#required' => TRUE,
      '#placeholder' => $site_domain,
      '#description' => $this->t('This domain will be used to create the dummy email for the useres who registered using mobile number'),
    ];

    // Load all available user roles.
    $roles = Role::loadMultiple();
    $role_options = [];
    foreach ($roles as $role_id => $role) {
      $role_options[$role_id] = $role->label();
    }

    $form['fieldset_wab_api'] = [
      '#type' => 'details',
      '#title' => $this->t('Whatsapp Business API Settings'),
      '#open' => TRUE,
      '#attributes' => [
        'class' => ['non-collapsible'],
      ],
    ];

    $form['fieldset_wab_template'] = [
      '#type' => 'details',
      '#title' => $this->t('Whatsapp Business Template Settings'),
      '#open' => TRUE,
      '#attributes' => [
        'class' => ['non-collapsible'],
      ],
    ];

    $form['fieldset_wab_api']['wab_access_token'] = [
      '#type' => 'textarea',
      '#required' => TRUE,
      '#title' => $this->t('WhatsApp Business Access Token'),
      '#description' => $this->t('WhatsApp Business access token, review https://developers.facebook.com/apps/'),
      '#default_value' => $this->config('whatsapp_business.settings')->get('wab_access_token'),
    ];

    $form['fieldset_wab_api']['wab_api_version'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Version'),
      '#placeholder' => $this->t('v21.0'),
      '#required' => TRUE,
      '#description' => $this->t('WhatsApp Business API Version'),
      '#default_value' => $this->config('whatsapp_business.settings')->get('wab_api_version'),
    ];

    $form['fieldset_wab_api']['wab_phone_number_id'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('Phone number ID'),
      '#description' => $this->t('WhatsApp Business Phone number ID'),
      '#default_value' => $this->config('whatsapp_business.settings')->get('wab_phone_number_id'),
    ];

    $form['fieldset_wab_api']['wab_account_id'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('WhatsApp Business Account ID'),
      '#description' => $this->t('WhatsApp Business Account ID, review https://developers.facebook.com/apps/'),
      '#default_value' => $this->config('whatsapp_business.settings')->get('wab_account_id'),
    ];

    $form['fieldset_wab_template']['wab_default_template'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('WhatsApp Business default template'),
      '#placeholder' => $this->t('hello_world'),
      '#description' => $this->t('WhatsApp Business default messeging template to use, review https://developers.facebook.com/apps/'),
      '#default_value' => $this->config('whatsapp_business.settings')->get('wab_default_template'),
    ];

    $form['fieldset_wab_template']['wab_default_template_lang_code'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('WhatsApp Business default template language code'),
      '#placeholder' => $this->t('en_US'),
      '#description' => $this->t('WhatsApp Business default messeging template to use, review https://developers.facebook.com/apps/'),
      '#default_value' => $this->config('whatsapp_business.settings')->get('wab_default_template_lang_code'),
    ];

    $form['fieldset_wab_template']['wab_default_template_variables'] = [
      '#type' => 'textarea',
      '#placeholder' => $this->t('{{ otp }}'),
      '#title' => $this->t('WhatsApp Business default template variables'),
      '#description' => $this->t('WhatsApp Business default template variables each in a separate line, review https://developers.facebook.com/apps/'),
      '#default_value' => $this->config('whatsapp_business.settings')->get('wab_default_template_variables'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    // @todo Validate the form here.
    // Example:
    // @code
    //   if ($form_state->getValue('example') === 'wrong') {
    //     $form_state->setErrorByName(
    //       'message',
    //       $this->t('The value is not correct.'),
    //     );
    //   }
    // @endcode
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->config('whatsapp_business.settings')
      ->set('wab_phone_number_id', $form_state->getValue('wab_phone_number_id'))
      ->set('wab_api_version', $form_state->getValue('wab_api_version'))
      ->set('wab_account_id', $form_state->getValue('wab_account_id'))
      ->set('wab_access_token', $form_state->getValue('wab_access_token'))
      ->set('wab_default_template', $form_state->getValue('wab_default_template'))
      ->set('wab_default_template_lang_code', $form_state->getValue('wab_default_template_lang_code'))
      ->set('wab_default_template_variables', $form_state->getValue('wab_default_template_variables'))
      ->set('wab_user_roles', $form_state->getValue('wab_user_roles'))
      ->set('otp_user_domain', $form_state->getValue('otp_user_domain'))
      ->save();
    parent::submitForm($form, $form_state);
  }

}
